/*!
 * Copyright (c) 2011, 2012 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 * \author Sergey Bostandzhyan, jin@dev.digitalstrom.org
 */

#ifndef CONNECTION_H_
#define CONNECTION_H_

#include <digitalSTROM/dsm-api-v2/dsm-api.h>

#include "errormessage.h"

class Connection
{
public:
  Connection(ErrorMessage& errorMessage)
    : m_opened(false),
      m_dsmApiHandle(NULL),
      m_errorMessage(errorMessage)
    {}
  
  bool open(const std::string& connection) {
    m_dsmApiHandle = DsmApiInitialize();
    if (!m_dsmApiHandle) {
      m_errorMessage.setError("Couldn't init dsmapi connection", RETCODES::ERR_DSM_LIB_INIT);
      return false;
    }

    int result = DsmApiOpen(m_dsmApiHandle, connection.c_str(), 0);
    if (result < 0) {
      m_errorMessage.setError("Couldn't open dsmapi connection", RETCODES::ERR_DSM_LIB_INIT);
      return false;
    }

    m_opened = true;
    return true;
  }
  
  void* handle() {
    return m_dsmApiHandle;
  }
  
  bool isOpen() const {
    return m_opened;
  }
  
  void close() {
    if (m_opened) {
      DsmApiCleanup(m_dsmApiHandle);
      DsmApiClose(m_dsmApiHandle);
      m_opened = false;
      m_dsmApiHandle = NULL;

      init();
    }
  }
  
  void init() {
    m_opened = false;
    m_dsmApiHandle = NULL;
  }

  bool getLinkState() {
    bool link = false;
    if (m_opened) {
      DsmApiGetLinkState(m_dsmApiHandle, &link);
    }
    return link;
  }
  
private:
  bool m_opened;
  void* m_dsmApiHandle;

  ErrorMessage& m_errorMessage;
};

#endif /* CONNECTION_H_ */
