/*!
 * Copyright (c) 2010, 2012 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 * \author Sergey Bostandzhyan, jin@dev.digitalstrom.org
 */

#ifndef DSM_H_
#define DSM_H_

#include <inttypes.h>
#include <string>
#include <list>

#include <digitalSTROM/dsuid.h>
#include <digitalSTROM/dsm-api-v2/dsm-api.h>

class FirmwareImageInfo;
class ErrorMessage;
class Connection;

typedef struct {
  dsuid_t dsuid;
  uint8_t type;
} busMemberDevice_t;

/*!
 * \class DSF
 * \brief dSF abstraction; takes care of all the ds485 communication
 */
class DSF
{
public:
  static bool initializeTransfer(Connection* connection, 
                                 dsuid_t& dsuid,
                                 const FirmwareImageInfo& firmwareImageInfo);
  static int sendChunk(Connection* connection, const dsuid_t& dsuid,
                       uint8_t* data, uint32_t packetNumber,
                       uint32_t* expectedPacketNumber);

  static bool verifyImage(Connection* connection, dsuid_t dsuid, uint32_t DeviceId);
  static bool startUpdate(Connection* connection, dsuid_t dsuid, uint32_t DeviceId);

  static int getBusMembers(Connection* connection, std::list<busMemberDevice_t> &device_list, bool dSMDsuidOnly = false);
  static bool listDevices(Connection* connection, std::list<std::string>& devices, bool dSMDsuidOnly);
  static bool getDeviceCount(Connection* connection, int *count);

  static bool getVersion(Connection* connection,
                         dsuid_t& dsuid,
                         uint32_t* hwVersion,
                         uint32_t* armSwVersion,
                         uint32_t* dspSwVersion,
                         uint32_t* boosterArmSwVersion,
                         uint16_t* apiVersion,
                         std::string& name);

  static void dumpBuffer(uint8_t* data, uint32_t length);

  static bool parseDSUIDFromString(dsuid_t* dsuid, const std::string& dsuidString);

  static const uint32_t CHUNK_SIZE;

};

#endif /* DSM_H_ */
