/*!
 * Copyright (c) 2011, 2012 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 * \author Sergey Bostandzhyan, jin@dev.digitalstrom.org
 */

#ifndef DSFGROUP_H_
#define DSFGROUP_H_

#include <list>
#include <pthread.h>
#include <utility>

#include <boost/shared_ptr.hpp>

#include <digitalSTROM/dsuid.h>
#include "connection.h"

class FirmwareFile;

class DsfGroup
{
public:

  struct Dsf
  {
    dsuid_t dsuid;
    std::string dsuidString;
    bool ok;
    bool rebooted;
    bool expectReboot;
  };

  DsfGroup(Connection* connection, bool messages);
  ~DsfGroup();

  void add(const std::string& dsuid);
  void reduce();
  void restore();
  bool allOk();
  int size();
  int sizeRemaining();

  void containedDsuids(std::list<std::string>& dsfList);

  // - info during setup
  void prepareForNextPacket();
  int nextPacketNumber();

  // - actual dSF actions
  void removeUpToDateDsfs(FirmwareFile& firmwareFile, std::list<std::string>& uptoDateDsfs, bool removeFailed = false, bool verbose = true);
  int initializeTransfer(FirmwareFile& firmwareFile);
  int transferData(FirmwareFile& firmwareFile);


  int verifyData(uint32_t deviceId);
  int startUpdate(uint32_t deviceId);

  // returns true if all dSFs in group are online, provides list of online
  // device dsuids
  bool online();
  // waits for the dSFs in group to get online
  bool waitForDevices();

private:
  uint32_t m_nextPacketNo;
  pthread_mutex_t m_dsf_list_mutex;
  pthread_mutex_t m_dsf_full_list_mutex;
  size_t m_returned;

  std::list<boost::shared_ptr<DsfGroup::Dsf> > m_dsfs;
  std::list<boost::shared_ptr<DsfGroup::Dsf> > m_dsfs_full;

  Connection* m_connection;
  bool m_messages;

  static void busChangeCallback(void *arg, dsuid_t *dsuid, int flag);
  void handleBusChange(dsuid_t *dsuid, int flag);
  static void dataCallback(uint8_t errorCode, void *arg,
                           dsuid_t sourceId, dsuid_t destinationId,
                           uint32_t expectedPacketNo,
                           uint32_t receivedPacketNo);
  void handleDataCallback(uint8_t errorCode,
                          dsuid_t sourceId, dsuid_t destinationId,
                          uint32_t expectedPacketNo, uint32_t receivedPacketNo);
  void lockDsfList();
  void unlockDsfList();
  void lockDsfFList();
  void unlockDsfFList();
  void resetRebootFlags();
  bool allRebooted();
  void unregisterBusChangeCallback();
  void unregisterDataCallback();
  void unregisterStartUpdateCallback();
  void setExpectRebootFlag(std::string dsuid, bool flag);
};

#endif /* DSFGROUP_H_ */
