/*!
 * Copyright (c) 2010 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 */


#ifndef FIRMWAREIMAGEINFO_H_
#define FIRMWAREIMAGEINFO_H_

/*!
 * \file firmwareimageinfo.h
 * \brief Firmware Image Info
 */

enum DeviceId_t : uint32_t {
  DSP_DSM1x         = 0x00000001,
  ARM_DSM11         = 0x00000002,
  ARM_DSM12         = 0x00000003,
  ARM_DSM20         = 0x00000100,
  DSP_DSM20         = 0x00000101,
  ARM_DSF25         = 0x00000200,
  DSP_DSF25         = 0x00000201,
  BOOSTER_ARM_DSF25 = 0x00000202
};

/*!
 * \brief firmware Image info (header), taken from dSM
 */
struct FirmwareImageInfo
{
  uint8_t header[4];    //!< 'aizo'
  uint32_t headerSize; //!< header size incl. 'header'+'headerSize'; usually 512 bytes
  uint32_t manufacturer; //!< manufacturer: aizo = 0x04175FE
  uint32_t device;       //!< device: DeviceId_t
  uint32_t contentType;  //!< content type: Firmware = 1
  uint32_t contentSize;  //!< content size in bytes
  uint16_t contentCrc;   //!< content CRC-16
  uint32_t version;      //!< firmware version
};

/*!
 * \def FIRMWARE_HEADER_TAG(dw)
 * \brief the header start tag to recognize a dSM image by
 */
#define FIRMWARE_HEADER_TAG "aizo"

/*!
 * \def EXPLODE_UINT32(dw)
 * \brief explode a uint32 variable into a big endian byte uint8 array
 */
#define EXPLODE_UINT32(dw) \
  static_cast<uint8_t>(0xff & (dw >> 24)),\
  static_cast<uint8_t>(0xff & (dw >> 16)),\
  static_cast<uint8_t>(0xff & (dw >>  8)),\
  static_cast<uint8_t>(0xff & (dw >>  0))

/*!
 * \def EXPLODE_UINT16(dw)
 * \brief explode a uint16 variable into a big endian byte uint8 array
 */
#define EXPLODE_UINT16(w) \
  static_cast<uint8_t>(0xff & (w >> 8)),\
  static_cast<uint8_t>(0xff & (w >> 0))

#endif /* FIRMWAREIMAGEINFO_H_ */
