/*!
 * Copyright (c) 2010, 2012 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 * \author Sergey Bostandzhyan, jin@dev.digitalstrom.org
 */

#ifndef FIRMWAREUPGRADE_H_
#define FIRMWAREUPGRADE_H_

#include <string>
#include <list>

#include "digitalSTROM/dsuid.h"
#include "retcodes.h"

class FirmwareFile;
class Connection;
class DsfGroup;
class MessageWriter;

/*!
 * \class FirmwareUpgrade
 * \brief Implements the firmware upgrade for dSMs
 *
 * \see FirmwareFile
 * \see DSM
 */
class FirmwareUpgrade
{
public:
  FirmwareUpgrade();
  int run(int argc, char** argv);


private:
  void usage();

  int updateFirmware();
  int listDevices();
  int getDSFVersion();
  int broadcastUpdate();

  uint32_t getFirmwareType(std::string fileName);
  void sortFirmwareFiles(std::list<std::string>& fileNameList);
  int upgradeGroup(DsfGroup* dsmGroup);
  int upgradeGroupFirmware(DsfGroup* dsmGroup, std::string fileName,
                           std::list<std::string>& upToDateDsms,
                           std::list<std::string>& updatedDsms);
  bool executeUpgrade(DsfGroup* group, FirmwareFile *fwFile);
  void updateSummary(DsfGroup* group, time_t *start, time_t *end,
                     std::list<std::string>& updatedDsms);
  bool parseArguments(int argc, char** argv);

  RETCODES::ReturnValue transferData(FirmwareFile& firmwareFile, dsuid_t& dsuid);
  
  static const std::string DEFAULT_CONNECTION;
  static const int MAX_TRIES = 2;

  const char* m_appName;

  // config
  std::list<std::string> m_fileNames;
  std::string m_connectionString;
  std::string m_dsfDSUID;
  std::string m_msgFile;

  bool m_scanMode;
  bool m_versionMode;
  bool m_broadcastMode;
  bool m_debug;
  bool m_pretend;
  bool m_force; ///Upgrade even if version matches

  Connection* m_connection;
};

#endif /* FIRMWAREUPGRADE_H_ */
