#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <stdexcept>

#include "messagewriter.h"

MessageWriter *MessageWriter::m_instance = NULL;

MessageWriter *MessageWriter::createInstance(std::string filename)
{
  if (m_instance == NULL) {
    m_instance = new MessageWriter(filename);
  }
  return m_instance;
}

MessageWriter *MessageWriter::getInstance()
{
  if (m_instance == NULL) {
    throw std::runtime_error("MessageWriter class not initialized");
  }
  return m_instance;
}

MessageWriter::MessageWriter(std::string filename) :
        m_filename(filename)
{
  int ret = pthread_mutex_init(&m_msgMutex, NULL);
  if (ret != 0) {
    throw std::runtime_error("could not initialize message mutex!");
  }
}

MessageWriter::~MessageWriter()
{
  pthread_mutex_destroy(&m_msgMutex);
  m_instance = NULL;
}

void MessageWriter::msg(std::string message)
{
  if (m_filename.empty()) {
    return;
  }

  if (pthread_mutex_lock(&m_msgMutex) != 0) {
    printf("Could not lock message mutex for message \"%s\": %s\n",
        message.c_str(), strerror(errno));
    return;
  }

  char tempname[17] = "/tmp/dsmfwXXXXXX";
  int fd = mkstemp(tempname);
  if (fd == -1) {
    printf("Could not create temporary file %s: %s\n", tempname,
        strerror(errno));
    pthread_mutex_unlock(&m_msgMutex);
    return;
  }
  ssize_t bytes = write(fd, message.c_str(), message.size());
  close(fd);
  if (bytes < 0) {
    printf("Error at write message: %s\n", strerror(errno));
    pthread_mutex_unlock(&m_msgMutex);
    return;
  } else if ((size_t) bytes != message.size()) {
    printf("Incomplete write for message: %s\n", message.c_str());
    pthread_mutex_unlock(&m_msgMutex);
    return;
  }

  if (rename(tempname, m_filename.c_str()) != 0) {
    printf("Could not apply message to file %s: %s\n", m_filename.c_str(),
        strerror(errno));
    unlink(tempname);
  }

  pthread_mutex_unlock(&m_msgMutex);
}
