/*!
 * Copyright (c) 2010 aizo ag, Switzerland
 *
 * \author Johannes Winkelmann, johannes.winkelmann@aizo.com
 */

#ifndef _RETCODES_H_
#define _RETCODES_H_

/*!
 * \namespace RETCODES
 * \brief namespace to wrap symbolic return values
 */
namespace RETCODES {

// as we usually return the number of dSMs manipulated, we split
// the range of exit codes into to partitions, with the error 
// codes starting at 128; theoretically, we could go as low as 64,
// as there can't be more than 63 dSMs on the bus
#define RET_OFFSET 128

/*!
 * \enum ReturnValue
 * \brief Symbolic return codes
 */
enum ReturnValue {
  SUCCESS,                   /*!< successful execution                  */

  ERR_GENERAL = RET_OFFSET,  /*!< general error                         */
  ERR_ARG,                   /*!< argument error                        */

  ERR_FILE_OPEN,             /*!< error while opening file              */
  ERR_FILE_IO,               /*!< file i/o error                        */

  ERR_DSM_LIB_INIT,          /*!< error initializing ds485 lib          */
  ERR_DSM_CONNECT,           /*!< error connecting to the dSM           */
  ERR_DSM_IMG_INIT_TRANSFER, /*!< error initializing the image transfer */
  ERR_DSM_IMG_TRANSFER,      /*!< error during the transfer             */
  ERR_DSM_IMG_VERIFY,        /*!< image verification returned an error  */
  ERR_DSM_IMG_UPDATE,        /*!< error in actual flash update          */
  ERR_DSM_IMG_RESTART_NEEDED,/*!< dSM wants us to start over            */

  ERR_MISSED_UPDATE          /*!< not all dSMs have been updated        */
};

}

#endif /* _RETCODES_H_ */
